# Copyright (c) 2014-2017 Pivotal Software, Inc. All rights reserved.
#
# Unauthorized use, copying or distribution of this source code via any
# medium is strictly prohibited without the express written consent of
# Pivotal Software, Inc.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
# EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
# MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
# IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
# CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
# TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
# SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

module PcfPause
  class OpsManInfo
    attr_reader :url, :token

    def initialize(url, username, password)
      @url = url
      @token = auth_header(username, password)
    end

    def get_products
      get_info 'deployed/products/'
    end

    def get_manifest(product_id)
      get_info "deployed/products/#{product_id}/manifest"
    end

    def get_vm_credentials(product_id)
      get_info "deployed/products/#{product_id}/vm_credentials"
    end

    def get_vm_names(product_id)
      manifest = get_manifest(product_id)
      key = if manifest.has_key?('jobs') # 1.7 manifest
              'jobs'
            else
              'instance_groups'
            end
      manifest[key].map { |vm| vm['name'] }
    end

    def get_director_manifest
      get_info 'deployed/director/manifest'
    end

    def get_product_id(type)
      get_products.detect { |product_info| product_info['type'] == type }['guid']
    end

    private

    def get_info(path)
      get_request = Net::HTTP::Get.new(File.join('/api/v0/', path))
      get_request['Authorization'] = token
      response = http_client.request(get_request)
      fail("Unexpected response: #{response.inspect}") unless response.code == '200'
      JSON.parse(response.body)
    end

    def http_client
      uri = URI.parse(url)
      Net::HTTP.new(uri.host, uri.port).tap do |http|
        http.use_ssl = true if url =~ /https/
        http.verify_mode = OpenSSL::SSL::VERIFY_NONE
        http.read_timeout = 300
      end
    end

    def auth_header(username, password)
      CF::UAA::TokenIssuer.new(URI.join(url, 'uaa'), 'opsman', '').tap do |issuer|
        issuer.skip_ssl_validation = true
      end.owner_password_grant(username, password).auth_header
    end
  end
end
